
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2003 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author(s): Josh Wilsdon <josh@wizard.ca>
 *
 * CVS Id: $Id: write_spam_log.c,v 1.1.1.1 2004/07/30 22:49:45 josh Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

/* Standard C includes */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/wait.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <sys/stat.h>
#include <signal.h>
#include <string.h>

#include <liblm.h>

/* magic-smtpd includes */
#include <magic-smtpd/smtp-limits.h>
#include <common/stopwatch.h>
#include <common/spam_log.h>
#include <magic-smtpd/config_qmail.h>
#include <common/write_spam_log.h>

#ifdef USE_LM_DATABASE

#include <postgresql/libpq-fe.h>
#include <common/postgresql.h>
#include <common/queries.h>
extern PGconn *db;

#endif

extern spam_log_t spam_log;

/* XXX
 *
 * NOTE: this is a huge ugly mess that should go away as soon as possible.
 *
 */

int writeSpamLog(int logtodb, char *insubject)
{
#ifdef USE_LM_DATABASE
    PGresult *result;
    char from[(FROM_LOG_MAX * 2) + 1];
    char user[(USERNAME_LOG_MAX * 2) + 1];
    char subject[(SUBJECT_LOG_MAX * 2) + 1];
    char domain[(DOMAIN_LOG_MAX * 2) + 1];
    char info[(DETAILS_LOG_MAX * 2) + 1];
    char buffer[2048];
    int initeddb = 0;
#endif
    char timebuff[128];
    time_t now;
    size_t bytes;
    struct tm *time_data;
    spam_log_entry_t *ptr;

    if (insubject != NULL) {
        spam_log.subject = strdup(insubject);
        if (spam_log.subject == NULL) {
            syslog(LOG_ERR, "writeSpamLog(): out of memory\n");
            return (1);
        }
    } else {
        spam_log.subject = insubject /* NULL */;
    }

#ifndef USE_LM_DATABASE
    if (logtodb == 1) {
        syslog(LOG_DEBUG, "DB logging requested, but not supported.");
    }
#endif

#ifdef USE_LM_DATABASE
    if (logtodb == 1) {
        if (connectDB() != -1) {
            initeddb = 1;
        }
    }
#endif

    if (spam_log.entries <= 0) {
        return (0);
    }

    if ((spam_log.first == NULL) || (spam_log.last == NULL)) {
        /* corrupted list structure */
        return (1);
    }

    /* NOTE: the only documented "man 2 time" error condition is when the
     *  argument is outside our address space, which NULL will not be. */
    now = time(NULL);
    if (now == ((time_t) - 1)) {
        /* perror("time()"); */
        return (1);
    }
    time_data = localtime(&now);
    if (time_data == NULL) {
        /* perror("localtime()"); */
        /* perror("time()"); */
        return (1);
    }

    /* TODO: should we free time_data? */

    bytes =
        strftime(timebuff, sizeof(timebuff), "%Y-%m-%d %H:%M:%S", time_data);
    if (bytes == 0) {
        /* perror("strftime()"); */
        return (1);
    }

    ptr = spam_log.first;
    while (ptr != NULL) {
        if (ptr->rule != NULL) {

#ifdef USE_LM_DATABASE
            if (spam_log.subject == NULL) {
                subject[0] = '\0';
            } else {
                /* truncate if necessary */
                if (strlen(spam_log.subject) > SUBJECT_LOG_MAX) {
                    spam_log.subject[SUBJECT_LOG_MAX] = '\0';
                }
                if (logtodb == 1) {
                    PQescapeString(subject, spam_log.subject,
                                   strlen(spam_log.subject));
                }
            }
            if (spam_log.sender == NULL) {
                from[0] = '\0';
            } else {
                /* truncate if necessary */
                if (strlen(spam_log.sender) > FROM_LOG_MAX) {
                    spam_log.sender[FROM_LOG_MAX] = '\0';
                }
                if (logtodb == 1) {
                    PQescapeString(from, spam_log.sender,
                                   strlen(spam_log.sender));
                }
            }
            if (spam_log.domain == NULL) {
                domain[0] = '\0';
            } else {
                /* truncate if necessary */
                if (strlen(spam_log.domain) > DOMAIN_LOG_MAX) {
                    spam_log.domain[DOMAIN_LOG_MAX] = '\0';
                }
                if (logtodb == 1) {
                    PQescapeString(domain, spam_log.domain,
                                   strlen(spam_log.domain));
                }
            }
            if (spam_log.user == NULL) {
                user[0] = '\0';
            } else {
                /* truncate if necessary */
                if (strlen(spam_log.user) > USERNAME_LOG_MAX) {
                    spam_log.user[USERNAME_LOG_MAX] = '\0';
                }
                if (logtodb == 1) {
                    PQescapeString(user, spam_log.user, strlen(spam_log.user));
                }
            }
            if (ptr->info == NULL) {
                info[0] = '\0';
            } else {
                /* truncate if necessary */
                if (strlen(ptr->info) > DETAILS_LOG_MAX) {
                    ptr->info[DETAILS_LOG_MAX] = '\0';
                }
                if (logtodb == 1) {
                    PQescapeString(info, ptr->info, strlen(ptr->info));
                }
            }

            /* Variables we do not clean: 
             *
             * timebuff - is generated by us so we don't need to worry about that.
             * rule     - needs to match with a rule in the database to have brought us here so that is safe
             * score    - is an integer so also ignore that 
             *
             */

            if (logtodb == 1) {
                snprintf(buffer, sizeof(buffer), QUERY_INSERT_SPAM_LOG, user,
                         domain, from, (subject[0] == '\0') ? ' ' : '\'',
                         (subject[0] == '\0') ? "NULL" : subject,
                         (subject[0] == '\0') ? ' ' : '\'', ptr->rule, info,
                         ptr->score, timebuff,
                         (config_qmail.tcpremoteip != NULL) ? LM_STRING_BUFFER(config_qmail.tcpremoteip) : "magic-local",
                         (ptr->global == 1) ? "Y" : "N");
                if ((db != NULL)
                    && ((result = doquery(buffer)) != NULL)
                    && (PQresultStatus(result) == PGRES_COMMAND_OK)) {
                    /* successfully logged it */
                } else {
                    syslog(LOG_ERR, "Failed SpamLog Query: [%s]\n", buffer);
                }
            } else {
#endif
                // XXX: the following does not work. 
                // syslog(LOG_INFO, "SPAM_HIT_DATA: %s,%s,%s,%s,%s,%s,%d,%s,%s,%s", spam_log.user, spam_log.domain, spam_log.sender, spam_log.subject, ptr->rule, ptr->info, ptr->score, timebuff, LM_STRING_BUFFER (config_qmail.tcpremoteip), (ptr->global == 1) ? "GLOBAL" : "PERSONAL" );
#ifdef USE_LM_DATABASE
            }
#endif
        }

        ptr = ptr->next;
    }

#ifdef USE_LM_DATABASE
    if (initeddb == 1) {
        disconnectDB();
    }
#endif

    return (0);
}
